from ..schemas import WeatherForecast
from ..types import WeatherForecastData
from .parse_location_data import parse_location_data
from .parse_weather_data import parse_weather_data

__all__ = ["parse_weather_forecast_data"]


def parse_weather_forecast_data(
    weather_forecast_data: WeatherForecastData,
) -> WeatherForecast:
    """
    Parse `WeatherForecastData` JSON.

    Args:
        weather_forecast_data: A dict representing JSON of weather forecast data.

    Returns:
        A `WeatherForecast` schema.
    """
    location = parse_location_data(weather_forecast_data["lokasi"])
    weathers = [
        parse_weather_data(weather)
        for weather in weather_forecast_data["data"][0]["cuaca"][0]
    ]

    return WeatherForecast(location, weathers)



from typing import TypedDict

__all__ = [
    "EarthquakeData",
    "FeltEarthquakeData",
    "LatestEarthquakeData",
    "StrongEarthquakeData",
    "InfoFeltEarthquakeData",
    "InfoLatestEarthquakeData",
    "InfoStrongEarthquakeData",
    "LocationData",
    "OptionalLocationData",
    "WeatherData",
    "WeatherForecastData",
]


class EarthquakeData(TypedDict):
    Tanggal: str
    Jam: str
    DateTime: str
    Coordinates: str
    Lintang: str
    Bujur: str
    Magnitude: str
    Kedalaman: str
    Wilayah: str


class StrongEarthquakeData(EarthquakeData):
    Potensi: str


class FeltEarthquakeData(EarthquakeData):
    Dirasakan: str


class LatestEarthquakeData(EarthquakeData):
    Potensi: str
    Dirasakan: str
    Shakemap: str


class _StrongEarthquakeData(TypedDict):
    gempa: list[StrongEarthquakeData]


class InfoStrongEarthquakeData(TypedDict):
    Infogempa: _StrongEarthquakeData


class _FeltEarthquakeData(TypedDict):
    gempa: list[FeltEarthquakeData]


class InfoFeltEarthquakeData(TypedDict):
    Infogempa: _FeltEarthquakeData


class _LatestEarthquakeData(TypedDict):
    gempa: LatestEarthquakeData


class InfoLatestEarthquakeData(TypedDict):
    Infogempa: _LatestEarthquakeData


class LocationData(TypedDict):
    adm1: str
    adm2: str
    adm3: str
    adm4: str
    provinsi: str
    kotkab: str
    kecamatan: str
    desa: str
    lon: float
    lat: float
    timezone: str


class OptionalLocationData(LocationData):
    type: str


class WeatherData(TypedDict):
    datetime: str
    t: int
    tcc: int
    tp: float
    weather: int
    weather_desc: str
    weather_desc_en: str
    wd_deg: int
    wd: str
    wd_to: str
    ws: float
    hu: int
    vs: int
    vs_text: str
    time_index: str
    analysis_date: str
    image: str
    utc_datetime: str
    local_datetime: str


class _WeatherForecastData(TypedDict):
    lokasi: OptionalLocationData
    cuaca: list[list[WeatherData]]


class WeatherForecastData(TypedDict):
    lokasi: LocationData
    data: list[_WeatherForecastData]



from dataclasses import dataclass

__all__ = ["Location"]


@dataclass(slots=True)
class Location:
    """
    A schema used to store information about a geographic location.

    Attributes:
        admin_level_1: The administrative level 1 code, typically representing the province.
        admin_level_2: The administrative level 2 code, typically representing the city or district.
        admin_level_3: The administrative level 3 code, typically representing the subdistrict.
        admin_level_4: The administrative level 4 code, typically representing a more localized area (e.g., village or neighborhood).
        province: The name of the province or state.
        city: The name of the city or district.
        subdistrict: The name of the subdistrict (a subdivision of a city or district).
        village: The name of the village or rural area.
        longitude: The geographic longitude coordinate of the location.
        latitude: The geographic latitude coordinate of the location.
        timezone: The time zone identifier for the location, e.g., "Asia/Jakarta".
    """

    admin_level_1: str
    admin_level_2: str
    admin_level_3: str
    admin_level_4: str
    province: str
    city: str
    subdistrict: str
    village: str
    longitude: float
    latitude: float
    timezone: str



from ..schemas import Location
from ..types import LocationData

__all__ = ["parse_location_data"]


def parse_location_data(location_data: LocationData) -> Location:
    """
    Parse `LocationData` JSON.

    Args:
        location_data: A dict representing JSON of weather forecast data.

    Returns:
        A `Location` schema.
    """

    return Location(
        location_data["adm1"],
        location_data["adm2"],
        location_data["adm3"],
        location_data["adm4"],
        location_data["provinsi"],
        location_data["kotkab"],
        location_data["kecamatan"],
        location_data["desa"],
        float(location_data["lon"]),
        float(location_data["lat"]),
        location_data["timezone"],
    )



import datetime as dt
from dataclasses import dataclass

from .. import enums

__all__ = ["Weather"]


@dataclass(slots=True)
class Weather:
    """
    A schema used to store information about a weather.

    Attributes:
        datetime: The datetime of the weather data in UTC format.
        t: Temperature in degrees Celsius.
        tcc: Total cloud cover percentage (0-100%).
        tp: Precipitation amount in millimeters.
        weather: Weather condition code, corresponds to a predefined set of weather conditions.
        wd_deg: Wind direction in degrees (0-360), where 0° is North.
        wd: Wind direction from which the wind blows.
        wd_to: Wind direction the wind is blowing towards.
        ws: Wind speed in kilometers per hour (km/h).
        hu: Humidity percentage (0-100%).
        vs: Visibility in meters.
        time_index: Time index in the format `"x-y"`, where `x` is the hour start of the forecast, and `y` is the hour end of the forecast.
        analysis_date: The date when the weather data was generated (in UTC format).
        image: URL to an image representing the weather condition (e.g., an icon).
        utc_datetime: The UTC datetime when the weather data was recorded.
        local_datetime: The local datetime when the weather data was recorded.
    """

    datetime: dt.datetime
    t: int
    tcc: int
    tp: float
    weather: enums.Weather
    wd_deg: int
    wd: enums.Cardinal
    wd_to: enums.Cardinal
    ws: float
    hu: int
    vs: int
    time_index: str
    analysis_date: dt.datetime
    image: str
    utc_datetime: dt.datetime
    local_datetime: dt.datetime



from .parse_earthquake_data import parse_earthquake_data
from .parse_felt_earthquake_data import parse_felt_earthquake_data
from .parse_latest_earthquake_data import parse_latest_earthquake_data
from .parse_location_data import parse_location_data
from .parse_strong_earthquake_data import parse_strong_earthquake_data
from .parse_weather_data import parse_weather_data
from .parse_weather_forecast_data import parse_weather_forecast_data

__all__ = [
    "parse_earthquake_data",
    "parse_felt_earthquake_data",
    "parse_latest_earthquake_data",
    "parse_location_data",
    "parse_strong_earthquake_data",
    "parse_weather_data",
    "parse_weather_forecast_data",
]



from ..parsers import parse_weather_forecast_data
from ..schemas import WeatherForecast as WeatherForecastData
from .api import API

__all__ = ["WeatherForecast"]


class WeatherForecast(API):
    """
    Weather Forecast API Wrapper from BMKG API.
    """

    base_url = "https://api.bmkg.go.id"
    url = "/publik/prakiraan-cuaca"

    async def get_weather_forecast(self, region_code: str) -> WeatherForecastData:
        """
        Request weather forecast from weather forecast API.

        Args:
            region_code: The administrative region code (level IV) for a subdistrict or village in Indonesia. The code is formatted as `W.X.Y.Z` (e.g., `"11.01.01.2001"`). You can find the list of available region codes at https://kodewilayah.id.

        Returns:
            A `WeatherForecastData` schema.

        Examples:
            >>> import asyncio
            >>> from bmkg import WeatherForecast
            >>> async def main():
            ...     async with WeatherForecast() as weather_forecast:
            ...         weather_forecast_data = await weather_forecast.get_weather_forecast(
            ...             "11.01.01.2001"
            ...         )
            ...         print(weather_forecast_data)
            >>> asyncio.run(main())
            WeatherForecast(location=Location(admin_level_1=...)
        """
        async with self._session.get(
            self.url, params={"adm4": region_code}
        ) as response:
            return parse_weather_forecast_data(await response.json())



from .coordinate import Coordinate
from .earthquake import Earthquake
from .felt_earthquake import FeltEarthquake
from .latest_earthquake import LatestEarthquake
from .location import Location
from .shakemap import Shakemap
from .strong_earthquake import StrongEarthquake
from .weather import Weather
from .weather_forecast import WeatherForecast

__all__ = [
    "Coordinate",
    "Earthquake",
    "FeltEarthquake",
    "LatestEarthquake",
    "Location",
    "Shakemap",
    "StrongEarthquake",
    "Weather",
    "WeatherForecast",
]



from dataclasses import dataclass

from .location import Location
from .weather import Weather

__all__ = ["WeatherForecast"]


@dataclass(slots=True)
class WeatherForecast:
    """
    A schema used to store information about weather forecast.

    Attributes:
        location: location information.
        weathers: list of weather forecast data.
    """

    location: Location
    weathers: list[Weather]



from traceback import TracebackException
from types import TracebackType
from typing import Self

from aiohttp import ClientSession

__all__ = ["API"]


class API:
    """
    Base API class.
    """

    def __init__(self, session: ClientSession | None = None) -> None:
        if not hasattr(self, "base_url"):
            raise NotImplementedError(
                f"{self.__class__.__name__} must define 'base_url'."
            )

        self._session = session if session is not None else ClientSession(self.base_url)

    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(
        self,
        exc_type: Exception,
        exc_val: TracebackException,
        traceback: TracebackType,
    ) -> None:
        await self.close()

    async def close(self) -> None:
        await self._session.close()



from datetime import datetime

from .. import enums
from ..schemas import Weather
from ..types import WeatherData

__all__ = ["parse_weather_data"]


def parse_weather_data(weather_data: WeatherData) -> Weather:
    """
    Parse `WeatherData` JSON.

    Args:
        weather_data: A dict representing JSON of weather forecast data.

    Returns:
        A `Weather` schema.
    """

    return Weather(
        datetime.fromisoformat(weather_data["datetime"]),
        int(weather_data["t"]),
        int(weather_data["tcc"]),
        float(weather_data["tp"]),
        enums.Weather(weather_data["weather"]),
        int(weather_data["wd_deg"]),
        enums.Cardinal(weather_data["wd"]),
        enums.Cardinal(weather_data["wd_to"]),
        float(weather_data["ws"]),
        int(weather_data["hu"]),
        int(weather_data["vs"]),
        weather_data["time_index"],
        datetime.fromisoformat(weather_data["analysis_date"]),
        weather_data["image"],
        datetime.strptime(weather_data["utc_datetime"], "%Y-%m-%d %H:%M:%S"),
        datetime.strptime(weather_data["local_datetime"], "%Y-%m-%d %H:%M:%S"),
    )
